%       Please cite the following references:
%
%       [1] W. Liao, J. Chanussot, M. Dalla Mura, X. Huang, R. Bellens,
%           S. Gautama, W, Philips, "Taking Optimal Advantage of Fine Spatial 
%           Resolution: Promoting partial image reconstruction for the 
%           morphological analysis of very-high-resolution images",  IEEE 
%           Geoscience and Remote Sensing Magazine, vol. 5, no. 2, pp. 8-28, 2017.
%
%       [2] W. Liao, M. Dalla Mura, J. Chanussot, R. Bellens, W. Philips, 
%           "Morphological Attribute Profiles With Partial Reconstruction", 
%           IEEE Transactions on Geoscience and Remote Sensing, vol. 54, 
%           no. 3, pp. 1738-1756, 2016.
%
%       [3] R. Bellens, J. Chan, L. Martinez-Fonte, S. Gautama, F. Canters, 
%           W. Philips, "Improved classification of VHR images of urban areas 
%           using directional morphological profiles", IEEE Transactions on
%           Geoscience and Remote Sensing, vol. 46, no. 10, pp. 2803-2813, 2008.
%
%
%
%       Copyright notes
%       Author: Rik Bellens and Wenzhi Liao, IPI, Telin, Ghent University, Belgium
%       Date: 19/07/2016
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


classdef periodiclinestrel < vhgwstrel
    
    properties (Access = protected)
        V
    end
    
    methods (Access = public)
        
        function obj = periodiclinestrel(P,V)
            if V(2)<0, V = -V; end
            obj@vhgwstrel(P,strel('periodicline',P,V));
            obj.V = V;
        end
        
    end
    
    methods (Access = protected)
            
        
        function im = rotateimage(im,se,b)
            b = cast(b,class(im));
            [h,w] = size(im);
            
            fullim = generatefullim(im,se,b);
            [fullh,fullw] = size(fullim);
            
            newh = fullh*se.V(2)+se.V(1);
            neww = ceil(numel(fullim)/newh);
            
            im = [fullim(:); repmat(b,neww*newh-numel(fullim),1);];
            im = reshape(im,newh,neww)';
        end
        
        function im = unrotateimage(im,se,h,w)
            im = im';
            [newh, neww] = size(im);
            s = getboundarysize(se);
            fullh = h + 2*s(1);
            fullw = w + 2*s(2);
            im = im(1:(fullh*fullw));
            im = reshape(im,fullh,fullw);
            
        end
        
    end
    
end
            
            
